/*
 * Copyright (c) 1983 Regents of the University of California.
 * All rights reserved.  The Berkeley software License Agreement
 * specifies the terms and conditions for redistribution.
 */

#ifndef lint
static char sccsid[] = "@(#)init_disp.c	5.1 (Berkeley) 6/6/85";
#endif not lint

/*
 * Initialization code for the display package,
 * as well as the signal handling routines.
 */

#include "talk.h"
#include <signal.h>
#include <sgtty.h>

#if (SUN3_1 | PYRAMID | ULTRIX1_2)
#define sigmask(m)	(1 << ((m)-1))
#endif

static struct sgttyb old_t, new_t;
int onstop();

/* 
 * Set up curses, catch the appropriate signals,
 * and build the various windows.
 */
init_display()
{
	void sig_sent();
	struct sigvec sigv;

 	ioctl(0, TIOCGETP, &old_t);	/* save old tty settings */

	if (my_interfac == I_TALK) {	/* do some window junk */
		initscr();
		curses_initialized = 1;
		clear();
		refresh();
		noecho();
		crmode();
	}

	(void) sigvec(SIGTSTP, (struct sigvec *)0, &sigv);
	sigv.sv_mask |= sigmask(SIGALRM);
	(void) sigvec(SIGTSTP, &sigv, (struct sigvec *)0);
	signal(SIGINT, sig_sent);
	signal(SIGPIPE, sig_sent);
	/* curses takes care of ^Z */

	if (my_interfac == I_TALK) {	/* do more window junk */
		my_win.x_nlines = LINES / 2;
		my_win.x_ncols = COLS;
		my_win.x_win = newwin(my_win.x_nlines, my_win.x_ncols, 0, 0);
		scrollok(my_win.x_win, FALSE);

		his_win.x_nlines = LINES / 2 - 1;
		his_win.x_ncols = COLS;
		his_win.x_win = newwin(his_win.x_nlines, his_win.x_ncols,
		    my_win.x_nlines+1, 0);
		scrollok(his_win.x_win, FALSE);
		wclear(his_win.x_win);

		line_win = newwin(1, COLS, my_win.x_nlines, 0);
		box(line_win, '-', '-');
		wrefresh(line_win);
 	} else if (my_interfac == I_CHAT) {	/* change tty for 'chat' */
 		signal(SIGTSTP, onstop);	/* catch TSTP to reset tty */
 		new_t = old_t;
 		new_t.sg_flags &= ~ECHO;	/* turn off echo */
 		new_t.sg_flags |= CBREAK;	/* get chars when typed */
 		ioctl(0, TIOCSETP, &new_t);	/* make new tty settings */
 	}

	/* let them know we are working on it */
	current_state = "No connection yet";
}

/*
 * Trade edit characters with the other talk. By agreement
 * the first three characters each talk transmits after
 * connection are the three edit characters.
 */
set_edit_chars()
{
	char buf[3];
	int cc;
	struct sgttyb tty;
	struct ltchars ltc;
	
	ioctl(0, TIOCGETP, &tty);
	ioctl(0, TIOCGLTC, (struct sgttyb *)&ltc);
	my_win.cerase = tty.sg_erase;
	my_win.kill = tty.sg_kill;
	if (ltc.t_rprntc == (char) -1)
		my_win.redraw = '\022';  /* control R */
	else
		my_win.redraw = ltc.t_rprntc;
	if (ltc.t_werasc == (char) -1)
		my_win.werase = '\027';	 /* control W */
	else
		my_win.werase = ltc.t_werasc;
	buf[0] = my_win.cerase;
	buf[1] = my_win.kill;
	buf[2] = my_win.werase;
	cc = write(sockt, buf, sizeof(buf));
	if (cc != sizeof(buf) )
		p_error("Lost the connection");
	cc = read(sockt, buf, sizeof(buf));
	if (cc != sizeof(buf) )
		p_error("Lost the connection");
	his_win.cerase = buf[0];
	his_win.kill = buf[1];
	his_win.werase = buf[2];
	his_win.redraw = '\0';
	my_win.column = his_win.column = 0;
}

void
sig_sent()
{
	message("Connection closing. Exiting");
	quit();
}

/*
 * All done talking...hang up the phone and reset terminal thingy's
 */
quit()
{

	if (curses_initialized && my_interfac == I_TALK) {
		wmove(his_win.x_win, his_win.x_nlines-1, 0);
		wclrtoeol(his_win.x_win);
		wrefresh(his_win.x_win);
		endwin();
	}
#ifdef NOFLUSH
	else if (my_interfac == I_CHAT) {
		ioctl(2, TIOCSETP, &old_t);
	}
#endif

	if (invitation_waiting)
		send_delete();

	/*
	 * The ioctl below is ONLY required for the `chat' inteface, however
	 * it is used here to flush the input buffer.  If you dont want the
	 * input buffer flushed when exiting `talk', define NOFLUSH...
	 */

#ifndef NOFLUSH
	ioctl(2, TIOCSETP, &old_t);
#endif
	exit(0);
}

#define	BIT(_a)		(1<<((_a)-1))

onstop()	/* when using `chat' trap to here on a SIGTSTP to restore tty */
{
	int signo = SIGTSTP;

	ioctl(0, TIOCSETP, &old_t);	/* restore tty to origional settings */
	signal(SIGTSTP, SIG_DFL);
	(void) sigsetmask(sigblock(0) & ~BIT(signo));
	(void) kill(getpid(), SIGTSTP);

	/* Wait until we are started up again */

	signal(SIGTSTP, onstop);	/* put the trap back */
	(void) sigsetmask(sigblock(0) | BIT(signo));
	ioctl(0, TIOCSETP, &new_t);	/* change the tty for 'chat' again */
}
